/*
   ----------------         shattered certainties.pov           ------------------------
   ----------------               9. march 2018                 ------------------------
   ----------------       copyright by Norbert W. Kern          ------------------------
   ----------------      norbert-werner.kern@t-online.de        ------------------------

 	

a comment on the present conditio humana.

Norbert  

http://news.povray.org/povray.binaries.images/thread/%3C5aa69f8c%40news.povray.org%3E/
 
*/

#version 3.7;
 
//_________________________________________________________________________________  

#declare grass_         = 1;
#declare sma_plants     = 0;
#declare craneo_cells   = 0;

//_________________________________________________________________________________

#declare RAD = 4;  
                
global_settings {
        #declare a_g = 2.2;
        assumed_gamma a_g
        max_trace_level 5
        noise_generator 2
        #if(RAD > 0)
                radiosity {
                        pretrace_start 0.08
                        pretrace_end   0.04/RAD
                        count 20*RAD                
                        nearest_count RAD      
                        error_bound 2/RAD
                        low_error_factor 0.5      
                        recursion_limit 1      
                        gray_threshold 0  
                        minimum_reuse 0.015 
                        brightness 2.3      
                        adc_bailout 0.005
                        normal on
                        media off
                }
        #end
}        

#include "stdinc.inc"

#declare cam_loc = <-0.45+0.02,0.1-0.05,-0.29+0.03>;
#declare lookat = <0,-0.14+0.14,-0.1>;
#declare cam_angle = 67*image_width/image_height*0.75;
#declare cam_up = y;

camera {
        location cam_loc 
        right -x*image_width/image_height 
        up cam_up 
        look_at lookat 
        angle cam_angle
}

#declare SunAngleX         = 14;
#declare SunAngleX_Terrain = 12;
#declare SunAngleY         = -20; 
#declare Transform_Sun =
transform {
        rotate x*SunAngleX
        rotate y*SunAngleY
}

#declare T_Clear = texture {pigment {Clear} finish {ambient 0 diffuse 0}}

//---------------------------------------
// Sun and sky
//---------------------------------------

#declare C_SkyBlue_05 = rgb <209,224,183>/255-(<234,255,160>/255-<209,224,183>/255)*0.5;
#declare C_SkyBlue_06 = rgb <234,255,160>/255+(<234,255,160>/255-<209,224,183>/255)*0.5;
#declare C_Sun = rgb <255,174,37>/255;

#macro GammaColor (Color,Gamma)

rgb <pow (Color.red, Gamma), pow (Color.green, Gamma), pow (Color.blue, Gamma)>

#end

//---------------------------------------
// Sun
//---------------------------------------

light_source {
        -z*100000
        color C_Sun*10
        area_light <8000,0,0>, <0,8000,0>, 4, 4 adaptive 0 jitter circular orient
        transform {Transform_Sun}
}
//---------------------------------------
// Sky
//---------------------------------------

sky_sphere {
        pigment {
                gradient y 
                color_map {
                        [0.05   C_SkyBlue_06]                        
                        [0.6    C_SkyBlue_05]
                }
        }
}

//_________________________________________________________________________________   

#declare cam_mirror = 1;
#declare cam_ang = cam_angle+0.5;                                          
#declare back_dist = 100;                                                       // needed for planting from camera location
#declare cam_z = 0.5*image_width/image_height/tan (radians (cam_ang*0.5));      // the amount of camera zoom
#declare cam_ang_up = degrees (2*atan2 (0.5/cam_z, 1));                         // camera angle in cam_up direction                                               
#declare cam_a = cam_mirror*image_width/image_height;                           // camera aspect ratio
#declare cam_s = y;                                                             // camera sky vector
#declare cam_d = vnormalize (lookat-cam_loc);                                   // camera direction vector
#declare cam_r = vnormalize (vcross (cam_s, cam_d));                            // camera right vector
#declare cam_up = vnormalize (vcross (cam_d, cam_r));                           // camera up vector
#declare cam_dir = cam_d*cam_z;                                                 // direction vector scaled
#declare cam_right = cam_r*cam_a;                                               // right vector scaled

#declare fz = vlength (cam_dir);
#declare fx = vlength (cam_right)*0.5;
#declare fy = vlength (cam_up)*0.5; 

#macro OrientZ (p1, p2, cs)
        
#local nz = vnormalize (p2-p1);
#local nx = vnormalize (vcross (cs,nz)); 
#local ny = vcross (nz,nx);
matrix <nx.x,nx.y,nx.z,ny.x,ny.y,ny.z,nz.x,nz.y,nz.z,p1.x,p1.y,p1.z>
                  
#end

box { 
        <0,0,0>, <1,1,0.1>
        texture {
                pigment {image_map {jpeg "makecloud2_demo_sunset5" interpolate 2} translate y*0.32} //__________________________________________________________
	        finish {emission 1 diffuse 0 ambient 0}
        }
        translate <-0.5,-0.5,0>
        scale 2*<cam_mirror*-fx,fy,0.5>
        translate fz*z
        scale back_dist
        OrientZ (cam_loc, lookat, cam_up)
        no_shadow 
}

#declare cam_loc = <-0.45,0.1,-0.29>;
#declare lookat = <0,-0.14+0.02,-0.1>;
#declare cam_angle = 67*image_width/image_height*0.8;
#declare cam_up = y;

//_________________________________________________________________________________  

#declare f_path = 
function {
        pigment {
                gradient y 
                scale 1.6
                color_map {
                        [0.006  rgb 0]
                        [0.006  rgb 1]
                }
        }
}
                         
#declare ground_tex =
texture { 
        gradient y scale 1.6            
        texture_map {
                [0.0045 pigment {image_map {jpeg "dirtmap" interpolate 2} rotate x*90}
                        normal {bump_map {jpeg "dirtmap" interpolate 2} 0.8 rotate x*90} 
                        finish {ambient 0 diffuse 0.2 specular 0.1 roughness 0.003}
                        scale 0.025
                ]
                [0.006  pigment {image_map {jpeg "grassmap" gamma 3.4 interpolate 2} rotate x*90}
                        normal {bump_map {jpeg "grassmap" interpolate 2} 2 rotate x*90} 
                        finish {ambient 0 diffuse 0.6}
                        scale 0.01
                ]
        }
}

#declare hfA = 
height_field { 
        png "shore2_" 
        smooth       
        scale <1,0.1,1>      
        translate <-0.5,0,-0.5>
        rotate y*90
        texture {ground_tex}
}

#declare C_Mud = rgb <106,111,104>/255;
#declare C_Water = rgb <96,105,100>/255; 

#declare colWater = rgb <78,104,48>/255*0.7;
#declare water_mat =
material {
        texture {
                pigment {rgbt <colWater.red,colWater.green,colWater.blue,0.8>}
                normal {
                        pigment_pattern {
                                bozo 
                                color_map {
                                        [0.45   rgb 0]
                                        [0.55   rgb 1]
                                }
                        }
                        normal_map {
                                [0      waves frequency 400 bump_size 0.1 turbulence 0.1]
                                [1      function {
                                                f_noise3d (x*1,y*5,z*5)*0.8
                                                +f_noise3d (x*10,y*50,z*50)*0.2
                                                +f_noise3d (x*100,y*200,z*200)*0.2
                                        }
                                        bump_size 0.06
                                ]
                        }
                }
                finish {
                        ambient 0
                        diffuse 0.14
                        reflection {0.03, 1}
                }
        } 
        interior {
                ior 1.33 
                caustics 1
                fade_distance 0.0025
                fade_power 1001
                fade_color rgb <0.7,0.8,0.2>
        	media {
        	        method 3
        	  	absorption (1-C_Mud)*19     
        	  	samples 2, 2
        	}
        }
}

//_________________________________________________________________________________

#declare f_bump1 = function {pigment {spotted scale 0.1}}
#declare f_bump2 = function {pigment {spotted translate -0.5 scale 0.06}}

#declare a = seed (0);

#include "transforms.inc"
#debug "planting\n"
#declare rnd_y = vaxis_rotate (lookat-cam_loc, vcross (lookat-cam_loc, cam_up), 0.5*cam_angle*image_height/image_width);
#declare rnd_xy = vaxis_rotate (rnd_y, cam_up, 0.5*cam_angle);
#declare rnd_y2 = vaxis_rotate (lookat-cam_loc, vcross (lookat-cam_loc, cam_up), -0.5*cam_angle*image_height/image_width);
#declare rnd_xy2 = vaxis_rotate (rnd_y2, cam_up, -0.5*cam_angle);
#declare p1 = cam_loc+rnd_xy*0.001;
#declare p0 = cam_loc+rnd_xy2*0.001;
#declare view_area = box {p0, p1}

//_________________________________________________________________________________

#if (grass_)

#include "pond_grass1_file.inc"
 
#declare grass_scal = 0.0005*3;
#declare grass_dist = 0.003*0.5*3;
#declare grass_numbX = 1/grass_dist;
#declare grass_numbZ = 1/grass_dist;

#declare q = 0;
#declare p = 0;

#declare grass1_ = 
union {
        #declare n = 0;
        #while (n < grass_numbX)
                #declare m = 0;
                #while (m < grass_numbZ)                
                        #declare obj_x = (n+rand (a)*0.6)*grass_dist-0.5;
                        #declare obj_z = (m+rand (a)*0.6)*grass_dist-0.5;
                        #declare Norm = <0,0,0>;
                        #declare Inter = trace (hfA, <obj_x, 1e4, obj_z>, <0,-1,0>, Norm);                 
                        #if ((vlength (Norm) > 0) & (Inter.y > 0.0077))
                                #declare af =(1+rand (a))/1.5;
                                #declare Norm1 = <0,0,0>;
                                #declare Norm2 = <0,0,0>;
                                #declare Inter2 = trace (view_area, Inter, cam_loc-Inter, Norm1);
                                #declare Inter3 = trace (hfA, Inter,cam_loc-Inter,Norm2);
                                #declare p = p+1;
                                #if ((vlength (Norm1) > 0) & (vlength (Inter-cam_loc) > vlength (Inter-Inter2)) & (vlength (Norm2) = 0))
                                        object {
                                                SprWed1  
                                                hollow double_illuminate
                                                scale grass_scal*af 
                                                rotate <-90,rand (a)*360,0>
                                                rotate <f_bump2 (Inter.x,Inter.y,Inter.z).gray*70+(rand (a)-0.5)*15,f_bump1 (Inter.x,Inter.y,Inter.z).gray*360+(rand (a)-0.5)*20,0>
                                                Reorient_Trans (y, Norm)
                                                translate Inter-<0,0.0005,0>
                                                #declare q = q+1;
                                        }
                                #end
                        #end
                        #declare m = m+1;    
                #end
                #declare n = n+1;
                #if (mod (n, 50) = 0)
                        #debug concat (str (n,0,0)," grass rows placed.\n")
                #end
        #end
        #debug concat (str (q,0,0)," grass objects planted.\n")
        #debug concat (str (p,0,0)," grass objects looked to plant.\n")
}

#end

//_________________________________________________________________________________

object {hfA} 
object {hfA scale <1,1,-1> translate <0,0,-1>}
box {
        -1, 1
        hollow
        double_illuminate
        scale <1,0.0075,1>      
        translate <-0.5,0,-0.5>
        material {water_mat scale 0.05 rotate y*-30}
}
#if (grass_) 
        object {grass1_}
        object {grass1_ scale <1,1,-1> translate <0,0,-1>}         
#end
